@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
- *  liboaf: A library for accessing oafd in a nice way.
+ *  bonobo-activation: A library for accessing bonobo-activation-server.
  *
  *  Copyright (C) 1999, 2000 Red Hat, Inc.
  *  Copyright (C) 2000 Eazel, Inc.
@@ -20,11 +20,11 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  *  Author: Elliot Lee <sopwith@redhat.com>
- *
  */
 
 #include <config.h>
 #include "bonobo-activation-init.h"
+#include "bonobo-activation-client.h"
 
 #include "Bonobo_ActivationContext.h"
 #include "bonobo-activation-i18n.h"
@@ -213,7 +213,7 @@ bonobo_activation_object_directory_get (const char *username,
 }
 
 static char *bonobo_activation_od_ior = NULL;
-static int bonobo_activation_ior_fd = 1;
+static int   bonobo_activation_ior_fd = 1;
 static char *bonobo_activation_activate_iid = NULL;
 
 struct poptOption bonobo_activation_popt_options[] = {
@@ -260,6 +260,7 @@ cmdline_check (const BonoboActivationBaseServiceRegistry *registry,
 {
 	if (!strcmp (base_service->name, "IDL:Bonobo/ObjectDirectory:1.0")) {
 		*distance = 0;
+                g_error ("This path is totally unsupported / tested");
 		return g_strdup (bonobo_activation_od_ior?bonobo_activation_od_ior:getenv("BONOBO_ACTIVATION_OD_IOR"));
 	}
 
@@ -336,6 +337,16 @@ static BonoboActivationBaseServiceRegistry ac_registry = {
 
 #define STRMATCH(x, y) ((!x && !y) || (x && y && !strcmp(x, y)))
 
+static CORBA_Object
+local_re_check_fn (const char        *display,
+                   const char        *act_iid,
+                   gpointer           user_data,
+                   CORBA_Environment *ev)
+{
+        return bonobo_activation_internal_service_get_extended (
+                user_data, TRUE, ev);
+}
+
 static CORBA_Object
 local_activator (const BonoboActivationBaseService *base_service,
                  const char **cmd,
@@ -349,7 +360,9 @@ local_activator (const BonoboActivationBaseService *base_service,
 		|| STRMATCH (base_service->hostname, bonobo_activation_hostname_get ()))
 	    && (!base_service->domain
 		|| STRMATCH (base_service->domain, bonobo_activation_domain_get ()))) {
-		return bonobo_activation_server_by_forking (cmd, fd_arg, NULL, NULL, ev);
+		return bonobo_activation_server_by_forking (
+                        cmd, FALSE, fd_arg, NULL, NULL, base_service->name,
+                        local_re_check_fn, (gpointer)base_service, ev);
 	}
 
 	return CORBA_OBJECT_NIL;
@@ -433,10 +446,10 @@ bonobo_activation_get_popt_table_name (void)
  * @argv: array of strings containing the command-line 
  *        arguments of the program.
  *
- * Initializes liboaf. Should be called before any other call to 
- * the library.
+ * Initializes bonobo-activation. Should be called before any other
+ * call to the library.
  *
- * Return value: the ORB used by OAF.
+ * Return value: the ORB used by bonobo-activation.
  */
 CORBA_ORB
 bonobo_activation_init (int argc, char **argv)
@@ -456,7 +469,8 @@ bonobo_activation_init (int argc, char **argv)
 	for (i = 1; i < argc; i++) {
 		if (!strncmp
 		    ("--oaf-od-ior=", argv[i], strlen ("--oaf-od-ior="))) {
-			bonobo_activation_od_ior = argv[i] + strlen ("--oaf-od-ior=");
+			bonobo_activation_od_ior =
+                                g_strdup (argv[i] + strlen ("--oaf-od-ior="));
 		} else if (!strncmp
                            ("--oaf-ior-fd=", argv[i],
                             strlen ("--oaf-ior-fd="))) {
@@ -468,7 +482,7 @@ bonobo_activation_init (int argc, char **argv)
                            ("--oaf-activate-iid=", argv[i],
                             strlen ("--oaf-activate-iid="))) {
 			bonobo_activation_activate_iid =
-				argv[i] + strlen ("--oaf-activate-iid=");
+				g_strdup (argv[i] + strlen ("--oaf-activate-iid="));
                 } else if (!strcmp
                            ("--oaf-private", argv[i])) {
                         bonobo_activation_private = TRUE;
@@ -483,6 +497,7 @@ bonobo_activation_init (int argc, char **argv)
 CORBA_ORB
 bonobo_activation_orb_init (int *argc, char **argv)
 {
+        CORBA_Context def_ctx;
 	CORBA_Environment ev;
 	const char *hostname;
         const char *display;
@@ -498,7 +513,10 @@ bonobo_activation_orb_init (int *argc, char **argv)
 	g_assert (ev._major == CORBA_NO_EXCEPTION);
 
 	/* Set values in default context */
-	CORBA_ORB_get_default_context (bonobo_activation_orb, &bonobo_activation_context, &ev);
+	CORBA_ORB_get_default_context (bonobo_activation_orb, &def_ctx, &ev);
+        CORBA_Context_create_child (def_ctx, "activation", &bonobo_activation_context, &ev);
+	g_assert (ev._major == CORBA_NO_EXCEPTION);
+        CORBA_Object_release ((CORBA_Object) def_ctx, &ev);
 	g_assert (ev._major == CORBA_NO_EXCEPTION);
 
 	hostname = bonobo_activation_hostname_get ();
@@ -563,6 +581,8 @@ bonobo_activation_debug_shutdown (void)
                         bonobo_activation_context = CORBA_OBJECT_NIL;
                 }
 
+                bonobo_activation_release_corba_client ();
+
                 if (bonobo_activation_orb != CORBA_OBJECT_NIL) {
                         CORBA_ORB_destroy (bonobo_activation_orb, &ev);
                         if (ev._major != CORBA_NO_EXCEPTION) {
