@@ -37,10 +37,11 @@
 #include "activation-context-query.h"
 #include "activation-server-corba-extensions.h"
 
+#undef LOCALE_DEBUG
+
 #define Bonobo_LINK_TIME_TO_LIVE 256
 
-typedef struct
-{
+typedef struct {
 	Bonobo_ObjectDirectory obj;
 
 	char *hostname, *username, *domain;
@@ -57,8 +58,7 @@ typedef struct
 	Bonobo_CacheTime time_active_pulled;
 
 	guchar locked;
-}
-ChildODInfo;
+} ChildODInfo;
 
 static ChildODInfo *
 child_od_info_new (Bonobo_ObjectDirectory obj, CORBA_Environment * ev)
@@ -95,6 +95,30 @@ child_od_info_new (Bonobo_ObjectDirectory obj, CORBA_Environment * ev)
 	return NULL;
 }
 
+static void
+child_od_info_free (ChildODInfo *child, CORBA_Environment *ev)
+{
+	CORBA_Object_release (child->obj, ev);
+	CORBA_free (child->hostname);
+	CORBA_free (child->username);
+	CORBA_free (child->domain);
+
+        if (child->by_iid)
+                g_hash_table_destroy (child->by_iid);
+
+        if (child->list) {
+                CORBA_sequence_set_release (child->list, CORBA_TRUE);
+                CORBA_free (child->list);
+        }
+
+        if (child->active_servers)
+                g_hash_table_destroy (child->active_servers);
+        CORBA_free (child->active_server_list);
+
+        memset (child, 0xaa, sizeof (ChildODInfo));
+	g_free (child);
+}
+
 static void
 child_od_exception (ChildODInfo * child, CORBA_Environment * ev)
 {
@@ -103,14 +127,14 @@ child_od_exception (ChildODInfo * child, CORBA_Environment * ev)
 }
 
 static void
-child_od_update_active (ChildODInfo * child, CORBA_Environment * ev)
+child_od_update_active (ChildODInfo *child, CORBA_Environment *ev)
 {
 	int i;
 	Bonobo_ServerStateCache *cache;
 
-	cache = Bonobo_ObjectDirectory_get_active_servers (child->obj,
-							child->time_active_pulled,
-							ev);
+	cache = Bonobo_ObjectDirectory_get_active_servers (
+                child->obj, child->time_active_pulled, ev);
+
 	if (ev->_major != CORBA_NO_EXCEPTION) {
 		child_od_exception (child, ev);
 		return;
@@ -127,25 +151,50 @@ child_od_update_active (ChildODInfo * child, CORBA_Environment * ev)
 		child->time_active_pulled = time (NULL);
 		child->active_servers =
 			g_hash_table_new (g_str_hash, g_str_equal);
-		g_hash_table_freeze (child->active_servers);
 		for (i = 0; i < cache->_u.active_servers._length; i++)
 			g_hash_table_insert (child->active_servers,
 					     cache->_u.
 					     active_servers._buffer[i],
 					     GINT_TO_POINTER (1));
-		g_hash_table_thaw (child->active_servers);
 	} else
 		CORBA_free (cache);
 }
 
+static char *
+ac_CORBA_Context_get_value (CORBA_Context         ctx, 
+                            const char           *propname,
+                            CORBA_Environment    *ev)
+{
+        return activation_server_CORBA_Context_get_value (
+                ctx, propname,
+                ex_Bonobo_Activation_IncompleteContext, ev);
+}
+
+/*** App-specific servant structures ***/
+
+typedef struct {
+	POA_Bonobo_ActivationContext servant;
+
+	GSList *dirs;
+
+	int total_servers;
+
+	gint refs;		/* This is a use count, so we don't accidentally go
+				 * updating our server list and invalidating memory */
+	Bonobo_ActivationContext self;
+} impl_POA_Bonobo_ActivationContext;
+
+/*** Stub implementations ***/
+
 static void
-child_od_update_list (ChildODInfo * child, CORBA_Environment * ev)
+child_od_update_list (ChildODInfo *child, CORBA_Environment *ev)
 {
 	int i;
 	Bonobo_ServerInfoListCache *cache;
 
-	cache = Bonobo_ObjectDirectory_get_servers (child->obj,
-						 child->time_list_pulled, ev);
+	cache = Bonobo_ObjectDirectory_get_servers (
+                child->obj, child->time_list_pulled, ev);
+
 	if (ev->_major != CORBA_NO_EXCEPTION) {
 		child->list = NULL;
 		child_od_exception (child, ev);
@@ -169,61 +218,31 @@ child_od_update_list (ChildODInfo * child, CORBA_Environment * ev)
 
 		child->time_list_pulled = time (NULL);
 		child->by_iid = g_hash_table_new (g_str_hash, g_str_equal);
-		g_hash_table_freeze (child->by_iid);
 		for (i = 0; i < child->list->_length; i++)
 			g_hash_table_insert (child->by_iid,
 					     child->list->_buffer[i].iid,
 					     &(child->list->_buffer[i]));
-		g_hash_table_thaw (child->by_iid);
 	}
 
 	CORBA_free (cache);
 }
 
-static void
-child_od_info_free (ChildODInfo * child, CORBA_Environment * ev)
+static void 
+free_child_dirs (impl_POA_Bonobo_ActivationContext *servant,
+                 CORBA_Environment                 *ev)
 {
-	CORBA_Object_release (child->obj, ev);
-	CORBA_free (child->hostname);
-	CORBA_free (child->username);
-	CORBA_free (child->domain);
-	g_free (child);
+	GSList *l;
+
+	for (l = servant->dirs; l; l = l->next)
+                child_od_info_free (l->data, ev);
+        g_slist_free (servant->dirs);
+        servant->dirs = NULL;
 }
 
-
-static char *
-ac_CORBA_Context_get_value (CORBA_Context         ctx, 
-                            const char           *propname,
-                            CORBA_Environment    *ev)
-{
-        return activation_server_CORBA_Context_get_value (
-                ctx, propname,
-                ex_Bonobo_Activation_IncompleteContext, ev);
-}
-
-
-/*** App-specific servant structures ***/
-
-typedef struct
-{
-	POA_Bonobo_ActivationContext servant;
-	PortableServer_POA poa;
-
-	GSList *dirs;
-
-	int total_servers;
-
-	gint refs;		/* This is a use count, so we don't accidentally go
-				 * updating our server list and invalidating memory */
-	Bonobo_ActivationContext me;
-}
-impl_POA_Bonobo_ActivationContext;
-
-/*** Stub implementations ***/
-
 static void
-ac_update_list (impl_POA_Bonobo_ActivationContext * servant,
-		ChildODInfo * child, CORBA_Environment * ev)
+ac_update_list (impl_POA_Bonobo_ActivationContext *servant,
+		ChildODInfo                       *child,
+                CORBA_Environment                 *ev)
 {
 	int prev, new;
 
@@ -246,13 +265,14 @@ ac_update_list (impl_POA_Bonobo_ActivationContext * servant,
 }
 
 static ChildODInfo *
-ac_find_child_for_server (impl_POA_Bonobo_ActivationContext * servant,
-			  Bonobo_ServerInfo * server, CORBA_Environment * ev)
+ac_find_child_for_server (impl_POA_Bonobo_ActivationContext *servant,
+			  Bonobo_ServerInfo                 *server,
+                          CORBA_Environment                 *ev)
 {
-	GSList *cur;
+	GSList *l;
 
-	for (cur = servant->dirs; cur; cur = cur->next) {
-		ChildODInfo *child = cur->data;
+	for (l = servant->dirs; l; l = l->next) {
+		ChildODInfo *child = l->data;
 
 		if (CORBA_Object_is_nil (child->obj, ev) || !child->list)
 			continue;
@@ -267,7 +287,7 @@ ac_find_child_for_server (impl_POA_Bonobo_ActivationContext * servant,
 }
 
 static QueryExprConst
-ac_query_get_var (Bonobo_ServerInfo * si, const char *id, QueryContext * qctx)
+ac_query_get_var (Bonobo_ServerInfo *si, const char *id, QueryContext *qctx)
 {
 	ChildODInfo *child;
 	QueryExprConst retval;
@@ -303,11 +323,12 @@ ac_query_get_var (Bonobo_ServerInfo * si, const char *id, QueryContext * qctx)
  * impl_Bonobo_ActivationContext_activate - hairy implicit preconditions
  * exist. */
 static void
-ac_query_run (impl_POA_Bonobo_ActivationContext * servant,
-	      const CORBA_char * requirements,
-	      const Bonobo_StringList * selection_order,
-	      CORBA_Context ctx,
-	      Bonobo_ServerInfo ** items, CORBA_Environment * ev)
+ac_query_run (impl_POA_Bonobo_ActivationContext *servant,
+	      const CORBA_char                  *requirements,
+	      const Bonobo_StringList           *selection_order,
+	      CORBA_Context                      ctx,
+	      Bonobo_ServerInfo                **items,
+              CORBA_Environment                 *ev)
 {
 	int total, i;
 	GSList *cur;
@@ -397,13 +418,16 @@ ac_query_run (impl_POA_Bonobo_ActivationContext * servant,
 }
 
 static void
-ac_update_lists (impl_POA_Bonobo_ActivationContext * servant,
-		 CORBA_Environment * ev)
+ac_update_lists (impl_POA_Bonobo_ActivationContext *servant,
+		 CORBA_Environment                 *ev)
 {
 	GSList *cur;
 
-	if (servant->refs > 0)
+	if (servant->refs > 0) {
+                /* FIXME: what happens on re-enterency here ?
+                 * looks like this could get seriously out of date */
 		return;
+        }
 
 	for (cur = servant->dirs; cur; cur = cur->next)
 		ac_update_list (servant, cur->data, ev);
@@ -411,6 +435,131 @@ ac_update_lists (impl_POA_Bonobo_ActivationContext * servant,
 
 #define GET_SERVANT(s) ((impl_POA_Bonobo_ActivationContext *)(s))
 
+static GList *clients = NULL;
+static GList *locale_list = NULL;
+
+void
+activation_clients_cache_notify (void)
+{
+        GList *l;
+        GSList *notify = NULL, *l2;
+        CORBA_Environment ev;
+
+        CORBA_exception_init (&ev);
+
+        for (l = clients; l; l = l->next)
+                notify = g_slist_prepend (notify, CORBA_Object_duplicate (l->data, &ev));
+
+        for (l2 = notify; l2; l2 = l2->next) {
+                Bonobo_ActivationClient_resetCache (l2->data, &ev);
+                if (ev._major != CORBA_NO_EXCEPTION)
+                        clients = g_list_remove (clients, l2->data);
+
+                CORBA_Object_release (l2->data, &ev);
+                CORBA_exception_free (&ev);
+        }
+}
+
+void
+activation_clients_check (void)
+{
+        GList *l, *next;
+
+        for (l = clients; l; l = next) {
+                next = l->next;
+                if (ORBit_small_get_connection_status (l->data) ==
+                    ORBIT_CONNECTION_DISCONNECTED) {
+                        CORBA_Object_release (l->data, NULL);
+                        clients = g_list_delete_link (clients, l);
+                }
+        }
+}
+
+void
+add_initial_locales (void)
+{
+        const char *tmp;
+        char *tmp2, *lang, *lang_with_locale, *equal_char;
+        
+        lang_with_locale = NULL;
+        
+        tmp = g_getenv ("LANGUAGE");
+
+        if (!tmp)
+                tmp = g_getenv ("LANG");
+        
+        lang = g_strdup (tmp);
+        tmp2 = lang;
+
+        if (lang) {
+                /* envs can be in NAME=VALUE form */
+		equal_char = strchr (lang, '=');
+		if (equal_char)
+			lang = equal_char + 1;
+
+                /* check if the locale has a _ */
+                equal_char = strchr (lang, '_');
+                if (equal_char != NULL) {
+                        lang_with_locale = g_strdup (lang);
+                        *equal_char = 0;
+                }
+
+                if (lang_with_locale && strcmp (lang_with_locale, "")) {
+                        locale_list = g_list_prepend (locale_list, lang_with_locale);
+#ifdef LOCALE_DEBUG
+                        g_warning ("Init lang '%s'", lang_with_locale);
+#endif
+                }
+
+                if (lang && strcmp (lang, "")) {
+                        locale_list = g_list_prepend (locale_list, g_strdup (lang));
+#ifdef LOCALE_DEBUG
+                        g_warning ("Init lang(2) '%s'", lang);
+#endif
+                }
+        }
+
+        g_free (tmp2);
+}
+
+gboolean
+is_locale_interesting (const char *locale)
+{
+        return (g_list_find_custom (locale_list, locale,
+                                    (GCompareFunc)strcmp) != NULL);
+}
+
+static void
+impl_Bonobo_ActivationContext_addClient (PortableServer_Servant        servant,
+                                         const Bonobo_ActivationClient client,
+                                         const CORBA_char             *locales,
+                                         CORBA_Environment            *ev)
+{
+        char **localev;
+        gboolean new_locale = FALSE;
+        int i;
+        
+        localev = g_strsplit (locales, ",", 0);
+
+        for (i = 0; localev[i]; i++) {
+                if (!g_list_find_custom (locale_list, localev[i],
+                                         (GCompareFunc)strcmp)) {
+#ifdef LOCALE_DEBUG
+                        g_warning ("New locale '%s' (%d)!",
+                                   localev[i], g_list_length (locale_list));
+#endif
+                        locale_list = g_list_prepend (locale_list, g_strdup (localev[i]));
+                        new_locale = TRUE;
+                }
+        }
+        g_strfreev (localev);
+        
+        clients = g_list_prepend (clients, CORBA_Object_duplicate (client, ev));
+
+        if (new_locale)
+                bonobo_object_directory_reload ();
+}
+
 static Bonobo_ObjectDirectoryList *
 impl_Bonobo_ActivationContext__get_directories (PortableServer_Servant _servant,
                                                 CORBA_Environment     *ev)
@@ -493,16 +642,17 @@ impl_Bonobo_ActivationContext_remove_directory (PortableServer_Servant _servant,
 }
 
 static void
-ac_do_activation (impl_POA_Bonobo_ActivationContext * servant,
-		  Bonobo_ServerInfo * server,
-		  Bonobo_ActivationResult * out,
-		  Bonobo_ActivationFlags flags,
-		  const char *hostname,
-		  CORBA_Context ctx, CORBA_Environment * ev)
+ac_do_activation (impl_POA_Bonobo_ActivationContext *servant,
+		  Bonobo_ServerInfo                 *server,
+		  Bonobo_ActivationResult           *out,
+		  Bonobo_ActivationFlags             flags,
+		  const char                        *hostname,
+		  CORBA_Context                      ctx,
+                  CORBA_Environment                 *ev)
 {
+	int num_layers;
 	ChildODInfo *child;
 	Bonobo_ServerInfo *activatable;
-	int num_layers;
 
 	/* When doing checks for shlib loadability, we 
          * have to find the info on the factory object in case
@@ -587,10 +737,9 @@ ac_do_activation (impl_POA_Bonobo_ActivationContext * servant,
 	} else {
 		CORBA_Object retval;
 
-		retval =
-			Bonobo_ObjectDirectory_activate (child->obj, server->iid,
-						      servant->me, flags, ctx,
-						      ev);
+		retval = Bonobo_ObjectDirectory_activate (
+                        child->obj, server->iid, servant->self, flags, ctx, ev);
+
 		if (ev->_major == CORBA_NO_EXCEPTION) {
 			char tbuf[512];
 			out->res._d = Bonobo_ACTIVATION_RESULT_OBJECT;
@@ -607,12 +756,13 @@ ac_do_activation (impl_POA_Bonobo_ActivationContext * servant,
 
 
 static Bonobo_ActivationResult *
-impl_Bonobo_ActivationContext_activate (PortableServer_Servant _servant,
-                                        const CORBA_char * requirements,
-                                        const Bonobo_StringList * selection_order,
-                                        const Bonobo_ActivationFlags flags,
-                                        CORBA_Context ctx,
-                                        CORBA_Environment * ev)
+impl_Bonobo_ActivationContext_activate (
+        PortableServer_Servant       _servant,
+        const CORBA_char            *requirements,
+        const Bonobo_StringList     *selection_order,
+        const Bonobo_ActivationFlags flags,
+        CORBA_Context                ctx,
+        CORBA_Environment           *ev)
 {
         impl_POA_Bonobo_ActivationContext *servant = GET_SERVANT (_servant);
 	Bonobo_ActivationResult *retval = NULL;
@@ -620,18 +770,18 @@ impl_Bonobo_ActivationContext_activate (PortableServer_Servant _servant,
 	int i;
 	char *hostname;
 
-	hostname = ac_CORBA_Context_get_value (ctx, "hostname", ev);
 	ac_update_lists (servant, ev);
 
 	servant->refs++;
 
 	items = g_alloca (servant->total_servers *
-			    sizeof (Bonobo_ServerInfo *));
+                          sizeof (Bonobo_ServerInfo *));
 	ac_query_run (servant, requirements, selection_order, ctx, items, ev);
 
-	if (ev->_major != CORBA_NO_EXCEPTION) {
+	if (ev->_major != CORBA_NO_EXCEPTION)
 		goto out;
-        }
+
+	hostname = ac_CORBA_Context_get_value (ctx, "hostname", ev);
 
 	retval = Bonobo_ActivationResult__alloc ();
 	retval->res._d = Bonobo_ACTIVATION_RESULT_NONE;
@@ -647,9 +797,9 @@ impl_Bonobo_ActivationContext_activate (PortableServer_Servant _servant,
 	if (retval->res._d == Bonobo_ACTIVATION_RESULT_NONE)
 		retval->aid = CORBA_string_dup ("");
 
-      out:
 	g_free (hostname);
 
+ out:
 	servant->refs--;
 
 	return retval;
@@ -706,16 +856,16 @@ impl_Bonobo_ActivationContext_activate_async(PortableServer_Servant _servant,
 	if (retval->res._d == Bonobo_ACTIVATION_RESULT_NONE)
 		retval->aid = CORBA_string_dup ("");
 
-
         if (ev->_major != CORBA_NO_EXCEPTION) {
                 char *message;
                 g_free (hostname);
                 CORBA_free (retval);
                 servant->refs--;
                 
-                message = g_strconcat(_("Activation failed: "), CORBA_exception_id (ev), NULL);
-                Bonobo_ActivationCallback_report_activation_failed (callback_object,
-                                                                 message, ev);
+                message = g_strconcat (_("Activation failed: "),
+                                       CORBA_exception_id (ev), NULL);
+                Bonobo_ActivationCallback_report_activation_failed (
+                        callback_object, message, ev);
                 g_free (message);
                 return;
         }
@@ -723,7 +873,8 @@ impl_Bonobo_ActivationContext_activate_async(PortableServer_Servant _servant,
 	g_free (hostname);
 
         /* return the correct value back to the client */
-        Bonobo_ActivationCallback_report_activation_succeeded (callback_object, retval, ev);
+        Bonobo_ActivationCallback_report_activation_succeeded (
+                callback_object, retval, ev);
 }
 
 static Bonobo_ServerInfoList *
@@ -825,19 +976,19 @@ ac_aid_to_query_string (const CORBA_char *aid)
         char *domain_requirement;
 	BonoboActivationInfo *ainfo;
 
-        /* FIXME bugzilla.eazel.com 4659: this is completely broken.
-		   We activate by AID, not IID. */
+        /*
+         * FIXME bugzilla.eazel.com 4659: this is completely broken.
+         * We activate by AID, not IID.
+         */
         if (strncmp ("OAFIID:", aid, 7) == 0) {
                 tmp_aid = g_strconcat ("OAFAID:[", aid, "]", NULL);
                 ainfo = bonobo_activation_id_parse (tmp_aid);
                 g_free (tmp_aid);
-        } else {
+        } else
                 ainfo = bonobo_activation_id_parse (aid);
-        }
 
-	if (ainfo == NULL) {
+	if (!ainfo)
                 return NULL;
-	}
 
         iid_requirement = g_strconcat ("iid == \'", ainfo->iid, "\' ", NULL);
 
@@ -882,9 +1033,8 @@ ac_context_to_string_array (CORBA_Context context, char **sort_criteria,
         context_username = ac_CORBA_Context_get_value (context, "username", ev);
         context_hostname = ac_CORBA_Context_get_value (context, "hostname", ev);
         context_domain = ac_CORBA_Context_get_value (context, "domain", ev);
-	if (ev->_major != CORBA_NO_EXCEPTION) {
+	if (ev->_major != CORBA_NO_EXCEPTION)
                 return;
-        }
         
         sort_criteria[0] = g_strconcat ("username == \'", context_username, "\'", NULL);
         sort_criteria[1] = g_strconcat ("hostname == \'", context_hostname, "\'", NULL);
@@ -896,9 +1046,6 @@ ac_context_to_string_array (CORBA_Context context, char **sort_criteria,
         g_free (context_domain);
 }
 
-
-
-
 static Bonobo_ActivationResult *
 impl_Bonobo_ActivationContext_activate_from_id (PortableServer_Servant _servant,
                                                 const CORBA_char *aid,
@@ -913,10 +1060,8 @@ impl_Bonobo_ActivationContext_activate_from_id (PortableServer_Servant _servant,
         Bonobo_StringList selection_order;
 
 	ac_update_lists (servant, ev);
-        if (ev->_major != CORBA_NO_EXCEPTION) {
+        if (ev->_major != CORBA_NO_EXCEPTION)
                 return CORBA_OBJECT_NIL;
-        }
-
 
 	servant->refs++;
 
@@ -929,6 +1074,7 @@ impl_Bonobo_ActivationContext_activate_from_id (PortableServer_Servant _servant,
         ac_context_to_string_array (ctx, sort_criteria, ev);
         if (ev->_major != CORBA_NO_EXCEPTION) {
                 servant->refs--;
+                g_free (requirements);
                 return CORBA_OBJECT_NIL;
         }
 
@@ -936,11 +1082,9 @@ impl_Bonobo_ActivationContext_activate_from_id (PortableServer_Servant _servant,
         selection_order._buffer = sort_criteria;
         CORBA_sequence_set_release (&selection_order, CORBA_FALSE);
 
-        retval = impl_Bonobo_ActivationContext_activate (servant,
-                                                      requirements,
-                                                      &selection_order,
-                                                      flags,
-                                                      ctx, ev);        
+        retval = impl_Bonobo_ActivationContext_activate (
+                servant, requirements, &selection_order, flags, ctx, ev);
+
         g_free (sort_criteria[0]);
         g_free (sort_criteria[1]);
         g_free (sort_criteria[2]);
@@ -954,12 +1098,12 @@ impl_Bonobo_ActivationContext_activate_from_id (PortableServer_Servant _servant,
 
 static void
 impl_Bonobo_ActivationContext_activate_from_id_async (
-        PortableServer_Servant    _servant,
+        PortableServer_Servant     _servant,
         const CORBA_char          *aid,
-        Bonobo_ActivationFlags    flags, 
-        Bonobo_ActivationCallback callback_object,
-        CORBA_Context             ctx, 
-        CORBA_Environment        *ev)
+        Bonobo_ActivationFlags     flags, 
+        Bonobo_ActivationCallback  callback_object,
+        CORBA_Context              ctx, 
+        CORBA_Environment         *ev)
 {
         impl_POA_Bonobo_ActivationContext *servant = GET_SERVANT (_servant);
 	Bonobo_ActivationResult *retval;
@@ -974,8 +1118,8 @@ impl_Bonobo_ActivationContext_activate_from_id_async (
         requirements = ac_aid_to_query_string (aid);
         if (requirements == NULL) {
                 servant->refs--;
-                Bonobo_ActivationCallback_report_activation_failed (callback_object,
-                                                                 _("Could not parse AID"), ev);
+                Bonobo_ActivationCallback_report_activation_failed (
+                        callback_object, _("Could not parse AID"), ev);
                 return;
         }
 
@@ -986,8 +1130,8 @@ impl_Bonobo_ActivationContext_activate_from_id_async (
 		g_free (requirements);
                 servant->refs--;
                 message = g_strconcat (_("Could not parse context: "), CORBA_exception_id (ev), NULL);
-                Bonobo_ActivationCallback_report_activation_failed (callback_object,
-                                                                 message, ev);
+                Bonobo_ActivationCallback_report_activation_failed (
+                        callback_object, message, ev);
                 g_free (message);
                 return;
         }
@@ -996,11 +1140,9 @@ impl_Bonobo_ActivationContext_activate_from_id_async (
         selection_order._buffer = sort_criteria;
         CORBA_sequence_set_release (&selection_order, CORBA_FALSE);
 
-        retval = impl_Bonobo_ActivationContext_activate (servant,
-                                                      requirements,
-                                                      &selection_order,
-                                                      flags,
-                                                      ctx, ev);
+        retval = impl_Bonobo_ActivationContext_activate (
+                servant, requirements, &selection_order, flags, ctx, ev);
+
         g_free (sort_criteria[0]);
         g_free (sort_criteria[1]);
         g_free (sort_criteria[2]);
@@ -1010,16 +1152,16 @@ impl_Bonobo_ActivationContext_activate_from_id_async (
                 char *message;
                 servant->refs--;
                 message = g_strconcat (_("Could not activate server: "), CORBA_exception_id (ev), NULL);
-                Bonobo_ActivationCallback_report_activation_failed (callback_object,
-                                                                 message, ev);
+                Bonobo_ActivationCallback_report_activation_failed (
+                        callback_object, message, ev);
                 g_free (message);
                 return;
         }
 
         servant->refs--;
 
-        Bonobo_ActivationCallback_report_activation_succeeded (callback_object,
-                                                            retval, ev);
+        Bonobo_ActivationCallback_report_activation_succeeded (
+                callback_object, retval, ev);
 }
 
 
@@ -1041,6 +1183,7 @@ static POA_Bonobo_Unknown__epv impl_Bonobo_Unknown_epv = {
 
 static POA_Bonobo_ActivationContext__epv impl_Bonobo_ActivationContext_epv = {
 	NULL,			/* _private */
+	impl_Bonobo_ActivationContext_addClient,
 	impl_Bonobo_ActivationContext__get_directories,
 	impl_Bonobo_ActivationContext_add_directory,
 	impl_Bonobo_ActivationContext_remove_directory,
@@ -1060,22 +1203,54 @@ static POA_Bonobo_ActivationContext__vepv impl_Bonobo_ActivationContext_vepv = {
 	&impl_Bonobo_ActivationContext_epv
 };
 
-Bonobo_ActivationContext
-Bonobo_ActivationContext_create (PortableServer_POA poa, CORBA_Environment * ev)
+static impl_POA_Bonobo_ActivationContext *main_ac = NULL;
+
+void
+activation_context_init (PortableServer_POA     poa,
+                         Bonobo_ObjectDirectory dir,
+                         CORBA_Environment     *ev)
 {
-	Bonobo_ActivationContext retval;
-	impl_POA_Bonobo_ActivationContext *newservant;
 	PortableServer_ObjectId *objid;
+	impl_POA_Bonobo_ActivationContext *servant;
 
-	newservant = g_new0 (impl_POA_Bonobo_ActivationContext, 1);
-	newservant->servant.vepv = &impl_Bonobo_ActivationContext_vepv;
-	newservant->poa = poa;
-	POA_Bonobo_ActivationContext__init ((PortableServer_Servant) newservant,
-					 ev);
-	objid = PortableServer_POA_activate_object (poa, newservant, ev);
+	servant = g_new0 (impl_POA_Bonobo_ActivationContext, 1);
+
+        main_ac = servant;
+
+	servant->servant.vepv = &impl_Bonobo_ActivationContext_vepv;
+	POA_Bonobo_ActivationContext__init ((PortableServer_Servant) servant, ev);
+	objid = PortableServer_POA_activate_object (poa, servant, ev);
 	CORBA_free (objid);
-	retval = newservant->me =
-		PortableServer_POA_servant_to_reference (poa, newservant, ev);
 
-	return CORBA_Object_duplicate (retval, ev);
+	servant->self = PortableServer_POA_servant_to_reference (poa, servant, ev);
+
+        impl_Bonobo_ActivationContext_add_directory (servant, dir, ev);
+}
+
+void
+activation_context_shutdown (PortableServer_POA poa,
+                             CORBA_Environment *ev)
+{
+	PortableServer_ObjectId *oid;
+	impl_POA_Bonobo_ActivationContext *servant = main_ac;
+
+	oid = PortableServer_POA_servant_to_id (poa, servant, ev);
+	PortableServer_POA_deactivate_object (poa, oid, ev);
+	CORBA_free (oid);
+
+        main_ac = NULL;
+        
+        free_child_dirs (servant, ev);
+        CORBA_Object_release (servant->self, ev);
+
+        g_free (servant);
+}
+
+Bonobo_ActivationContext
+activation_context_get (void)
+{
+        if (!main_ac)
+                return CORBA_OBJECT_NIL;
+        else
+                return main_ac->self;
 }
